@@ -3,13 +3,14 @@
 from typing import Union
 from typing import cast
 
-from twnet_parser import packer
+from twnet_parser.packer import Unpacker
 from twnet_parser.pretty_print import PrettyPrint
 from twnet_parser.message_parser import MessageParser
 from twnet_parser.net_message import NetMessage
 from twnet_parser.ctrl_message import CtrlMessage
 from twnet_parser.chunk_header import ChunkHeader, ChunkFlags
 from twnet_parser.msg_matcher.control7 import match_control7
+from twnet_parser.constants import NET_MAX_SEQUENCE
 
 from twnet_parser.external.huffman import huffman
 
@@ -41,12 +42,52 @@ class PacketFlags6(PrettyPrint):
         self.connless = False
 
 class PacketHeader(PrettyPrint):
-    def __init__(self) -> None:
-        self.flags: PacketFlags7 = PacketFlags7()
-        self.size: int = 0
-        self.ack: int = 0
-        self.token: bytes = b'\xff\xff\xff\xff'
-        self.num_chunks: int = 0
+    def __init__(
+            self,
+            flags: PacketFlags7 = PacketFlags7(),
+            ack: int = 0,
+            token: bytes = b'\xff\xff\xff\xff',
+            num_chunks: int = 0
+    ) -> None:
+        self.flags: PacketFlags7 = flags
+        self.ack: int = ack % NET_MAX_SEQUENCE
+        self.token: bytes = token
+        self.num_chunks: int = num_chunks
+
+    def pack(self) -> bytes:
+        """
+        Generate 7 byte teeworlds 0.7 packet header
+        based on the current instance variable
+        values.
+
+        The layout is as follows
+        6bit flags, 2bit ack
+        8bit ack
+        8bit chunks
+        32bit token
+
+        ffffffaa
+        aaaaaaaa
+        NNNNNNNN
+        TTTTTTTT
+        TTTTTTTT
+        TTTTTTTT
+        TTTTTTTT
+        """
+        flags = 0
+        if self.flags.control:
+            flags |= PACKETFLAG7_CONTROL
+        if self.flags.resend:
+            flags |= PACKETFLAG7_RESEND
+        if self.flags.compression:
+            flags |= PACKETFLAG7_COMPRESSION
+        if self.flags.connless:
+            flags |= PACKETFLAG7_CONNLESS
+        return bytes([ \
+            ((flags << 2)&0xfc) | ((self.ack>>8)&0x03), \
+            self.ack&0xff, \
+            self.num_chunks \
+        ]) + self.token
 
 class TwPacket(PrettyPrint):
     def __init__(self) -> None:
@@ -134,15 +175,16 @@ class PacketParser():
         i = 2
         if chunk_header.flags.vital:
             i += 1
-        msg_id: int = packer.unpack_int(data[i:])
+        unpacker = Unpacker(data[i:])
+        msg_id: int = unpacker.get_int()
         i += 1
         sys: bool = (msg_id & 1) == 1
         msg_id >>= 1
         msg: NetMessage
         if sys:
-            msg = MessageParser().parse_sys_message(msg_id, data[i:])
+            msg = MessageParser().parse_sys_message(msg_id, unpacker.get_raw())
         else:
-            msg = MessageParser().parse_game_message(msg_id, data[i:])
+            msg = MessageParser().parse_game_message(msg_id, unpacker.get_raw())
         msg.header = chunk_header
         return msg
 
